#include <ilcplex/ilocplex.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <iostream>
#include "Define_Read.h"
#include "Models.h"

using namespace std;

#define BILLION 1000000000L

typedef struct timespec timespec_t;

ILOSTLBEGIN


int main(int argc, char** argv) {
	if (argc <= 1) {
		exit(1);
	}
	IloEnv env;
	try {
		IloModel modelo(env);
		int T, J;
		IloNumArray a, C, h, isetup, istock;
		IloArray <IloNumArray> sc, st, ry, rw, b, d;
		IloArray <IloArray <IloNumArray> > rz;
		double FOtrad, DBtrad, GAPtrad, FO, DB, GAP, tempotrad, tempo;

		Define_Read(env,&argv[1],&T,&J,&a,&h,&b,&sc,&st,&d,&C,&isetup,&istock,&ry,&rw,&rz);

		timespec_t tStart, tEnd;
		clock_gettime(CLOCK_MONOTONIC, &tStart);

		IloArray <IloBoolVarArray> y, w; IloArray <IloArray <IloBoolVarArray> > z;

		CLSD_w(env,T,J,a,h,b,sc,st,d,C,isetup,istock,&z,&y,&w,&modelo);

		IloCplex cplex(env);

		cplex.extract(modelo);
		cplex.exportModel("saida.lp");
		cplex.setParam(IloCplex::TiLim, 3600);
		cplex.setParam(IloCplex::TreLim, 120000);

		for(int j = 0; j < J; ++j) {
			for(int t = 0; t < T; ++t) {
				cplex.setPriority(w[j][t],2);
			}
		}

		if(!cplex.solve()) {
			env.error() << "Não se pode resolver!" << endl;
			throw(-1);
		}
		IloInt nodes, iter;
		nodes = cplex.getNnodes();
		iter = cplex.getNiterations();
		FO = cplex.getObjValue();
		DB = cplex.getBestObjValue();
		GAP = 100*(FO -  DB)/FO;
		clock_gettime(CLOCK_MONOTONIC, &tEnd);
		tempo = (tEnd.tv_sec - tStart.tv_sec) + (double)(tEnd.tv_nsec - tStart.tv_nsec)/BILLION;
		cout << "FO = " << FO << " DB = " << DB << " GAP = " << GAP << " TIME = " << tempo << endl;

		FILE *res = fopen("Nodes_New_form.txt", "a+");
		fprintf(res, "%s \t %.2lf \t %.2lf \t %d \t %d\n", argv[2], FO, tempo, nodes, iter);
		fclose(res);

	}
	catch(IloException& ex) {
		cerr << "ERRO: " << ex << endl;
	}
	catch(...) {
		cerr << "ERRO!" << endl;
	}
	env.end();
	return 0;
}
